from __future__ import absolute_import

import sys, os, json
from catalyze import output

FILE_PATH = "./.git/catalyze-config.json"

def read_settings(required = True):
    if os.path.isdir("./.git"):
        if os.path.isfile(FILE_PATH):
            with open(FILE_PATH, 'r') as file:
                return json.load(file)
        else:
            if required:
                output.error("No Catalyze environment associated with this local repo. Run \"catalyze associate\" first.")
            return None
    elif required:
        output.error("No git repo found in the current directory.")
    else:
        return None

def save_settings(settings):
    with open(FILE_PATH, 'w') as file:
        json.dump(settings, file)

def clear_settings():
    if os.path.isdir("./.git"):
        if os.path.isfile(FILE_PATH):
            os.remove(FILE_PATH)



from __future__ import absolute_import

cli = None

def init_cli():
    import click
    from . import config, output

    @click.group("catalyze")
    @click.option("--baas-host", help = "Alternate BaaS API URL")
    @click.option("--paas-host", help = "Alternate PaaS API URL")
    @click.option("--username", help = "Catalyze Username")
    @click.option("--password", help = "Catalyze Password")
    @click.option("--skip-validation", is_flag = True, help = "Skip certificate validation")
    @click.version_option(version = config.version)
    def inner_cli(baas_host, paas_host, username, password, skip_validation):
        if baas_host is not None:
            config.baas_host = baas_host
            output.write("Overriding BaaS URL: " + config.baas_host)
        if paas_host is not None:
            config.paas_host = paas_host
            output.write("Overriding PaaS URL: " + config.paas_host)
        config.username = username
        config.password = password
        if skip_validation:
            config.behavior["skip_cert_validation"] = False
            output.write("Skipping cert validation, I hope this was intentional")

    global cli
    cli = inner_cli

    from catalyze.commands import \
        associate, backup, console, dashboard, db, environments, rake, redeploy, status, support_ids, users, variables, worker

def run():
    import catalyze.__main__



from __future__ import absolute_import

import sys, time
from catalyze import config, output

def poll_status(session, env_id, task_id):
    route = "%s/v1/environments/%s/tasks/%s" % (config.paas_host, env_id, task_id)
    while True:
        time.sleep(2)
        task = session.get(route, verify = True)
        if task["status"] not in ["scheduled", "queued", "started", "running"]:
            if task["status"] == "finished":
                return task["status"]
            else:
                output.write("")
                output.error("Error - ended in status '%s'." % (task["status"],))
        else:
            output.write(".", sameline = True)



from __future__ import absolute_import

import click
from catalyze import cli, client, git, project, output
from catalyze.helpers import environments, services

@cli.command(short_help = "Associates a local repository with an environment")
@click.argument("env_label")#, help = "The name of your environment (example = \"production\")")
@click.argument("service_label", required = False, default = None)#, help = "The name of the code service with which to associate your repository (example = \"app01\")")
@click.option("--remote", default = "catalyze", help = "The name of the git remote to be created.")
def associate(env_label, service_label, remote):
    """Associates the git repository in the current directory. This means that the service and environment IDs are stored locally, and a git remote is created (default name = "catalyze") so that code can be pushed, built, and deployed."""
    session = client.acquire_session()
    for env in environments.list(session):
        if env["data"]["name"] == env_label:
            settings = {
                    "token": session.token,
                    "user_id": session.user_id,
                    "environmentId": env["environmentId"]
                }
            code_services = [svc for svc in services.list(session, env["environmentId"]) if svc["type"] == "code"]
            selected_service = None
            if len(code_services) == 0:
                output.error("No code service found for \"%s\" environment (%s)" % (env_label, env["environmentId"]))
            elif service_label:
                for svc in code_services:
                    if svc["label"] == service_label:
                        selected_service = svc
                        break
                if selected_service is None:
                    output.error("No code service found with label '%s'. Labels found: %s" % \
                            (service_label, ", ".join([svc["label"] for svc in code_services])))
            elif len(code_services) > 1:
                output.error("Found multiple code services. Must pass one specifically to associate with. Labels found: " + \
                        ", ".join([svc["label"] for svc in code_services]))
            else:
                selected_service = code_services[0]

            if remote in git.remote_list():
                git.remote_remove(remote)
            git.remote_add(remote, selected_service["source"])
            settings["serviceId"] = selected_service["id"]
            project.save_settings(settings)
            output.write("\"%s\" remote added." % (remote,))
            return
    output.error("No environment with label \"%s\" found." % (env_label,))

@cli.command()
def disassociate():
    """Remove association with environment"""
    project.clear_settings()
    output.write("Association cleared.")



from __future__ import absolute_import

import click
from catalyze import cli, client, project, output
from catalyze.helpers import environments

@cli.command(short_help = "Retrieve your user ID")
def whoami():
    """Prompts for login, and prints out your ID so that you can be added to an environment by someone else."""
    session = client.acquire_session()
    output.write("user ID = " + session.user_id)

@cli.command(short_help = "Add a user to the environment")
@click.argument("user_id")
def adduser(user_id):
    """Adds another user to the associated environment. The ID required is found via 'catalyze whoami'."""
    settings = project.read_settings()
    session = client.acquire_session(settings)
    environments.add_user(session, settings["environmentId"], user_id)
    output.write("Added.")

@cli.command(short_help = "Remove a user from the environment")
@click.argument("user_id")
def rmuser(user_id):
    """Removes another user from the associated environment. The ID required is found via 'catalyze whoami'."""
    settings = project.read_settings()
    session = client.acquire_session(settings)
    environments.remove_user(session, settings["environmentId"], user_id)
    output.write("Removed.")

@cli.command(short_help = "List users for the environment")
def users():
    """Lists users in the associated environment."""
    settings = project.read_settings()
    session = client.acquire_session(settings)
    for user in environments.list_users(session, settings["environmentId"])["users"]:
        if user == settings["user_id"]:
            output.write("%s (you)" % (user,))
        else:
            output.write(user)



import subprocess

def remote_list():
    return subprocess.check_output(["git", "remote"]).split("\n")

def remote_add(name, url):
    subprocess.check_call(["git", "remote", "add", name, url])

def remote_remove(name):
    subprocess.check_call(["git", "remote", "remove", name])



from __future__ import absolute_import

import json
from catalyze import cli, client, project, output
from catalyze.helpers import environments, services

@cli.command(short_help = "Quick status readout")
def status():
    """Check the status of the environment and every service in it."""
    settings = project.read_settings()
    session = client.acquire_session(settings)
    env = environments.retrieve(session, settings["environmentId"])
    output.write("environment state: " + env["state"])
    codes = []
    noncodes = []
    for service in services.list(session, settings["environmentId"]):
        if service["type"] != "utility":
            if service["type"] == "code":
                codes.append("\t%s (size = %s, build status = %s, deploy status = %s)" % (service["label"], service["size"], service["build_status"], service["deploy_status"]))
            else:
                noncodes.append("\t%s (size = %s, image = %s, status = %s)" % (service["label"], service["size"], service["name"], service["deploy_status"]))
    for item in (codes + noncodes):
        output.write(item)



from __future__ import absolute_import

import click, json, time
from catalyze import cli, client, project, output
from catalyze.helpers import services, jobs, tasks
from datetime import datetime

def parse_date(date):
    return datetime.strptime(date, "%Y-%m-%dT%H:%M:%S")

@cli.group()
def backup():
    """Backup and restore services on demand"""

@backup.command("list", short_help = "List created backups")
@click.argument("service_label")#, help = "The name of the service.")
@click.option("--page", default = 1, help = "The page number to view")
@click.option("--page-size", default = 10, help = "The number of items to show per page")
def exec_list(service_label, page, page_size):
    """List all created backups for the service, sorted from oldest to newest."""
    settings = project.read_settings()
    session = client.acquire_session(settings)
    service_id = services.get_by_label(session, settings["environmentId"], service_label)
    raw_backups = services.list_backups(session, settings["environmentId"], service_id, page, page_size)
    backup_list = []
    for id, body in raw_backups.items():
        body["id"] = id
        backup_list.append(body)
    backup_list.sort(lambda a, b: int((parse_date(a["created_at"]) - parse_date(b["created_at"])).total_seconds()))
    if len(backup_list) > 0:
        for item in backup_list:
            output.write("%s %s (status = %s)" % (item["id"], item["created_at"], item["status"]))
        if len(backup_list) == page_size and page == 1:
            output.write("(for older backups, try with --page=2 or adjust --page-size)")
    elif page == 1:
        output.write("No backups created yet for this service.")

@backup.command(short_help = "Create a new backup")
@click.argument("service_label")#, help = "The name of the service.")
@click.option("--skip-poll", is_flag = True, default = False, help = "Just start the backup - don't poll.")
def create(service_label, skip_poll):
    settings = project.read_settings()
    session = client.acquire_session(settings)
    service_id = services.get_by_label(session, settings["environmentId"], service_label)
    task_id = services.create_backup(session, settings["environmentId"], service_id)
    print("Backup started (task ID = %s)" % (task_id,))
    if not skip_poll:
        output.write("Polling until backup finishes.")
        status = tasks.poll_status(session, settings["environmentId"], task_id)
        output.write("\nEnded in status '%s'" % (status,))

@backup.command(short_help = "Restore from a backup")
@click.argument("service_label")#, help = "The name of the service.")
@click.argument("backup_id")#, help = "The ID of the backup to be imported (acquired from 'catalyze backup list')")
@click.option("--skip-poll", is_flag = True, default = False, help = "Just start the restore - don't poll.")
def restore(service_label, backup_id, skip_poll):
    settings = project.read_settings()
    session = client.acquire_session(settings)
    service_id = services.get_by_label(session, settings["environmentId"], service_label)
    task_id = services.restore_backup(session, settings["environmentId"], service_id, backup_id)
    output.write("Restoring (task = %s)" % (task_id,))
    if not skip_poll:
        output.write("Polling until restore is complete.")
        status = tasks.poll_status(session, settings["environmentId"], task_id)
        output.write("\nEnded in status '%s'" % (status,))



from __future__ import absolute_import

import webbrowser

from catalyze import cli, config

@cli.command("dashboard")
def dashboard():
    """Open the Catalyze dashboard in your browser"""
    webbrowser.open(config.dashboard_url)



from __future__ import absolute_import

from catalyze import cli, client, config, project, output
from catalyze.helpers import services
import click

@cli.command(short_help = "Redeploy without pushing")
def redeploy():
    """Redeploy an environment's service manually."""
    settings = project.read_settings()
    session = client.acquire_session(settings)
    service_id = settings["serviceId"]
    output.write("Redeploying " + service_id)
    services.redeploy(session, settings["environmentId"], service_id)
    output.write("Redeploy successful, check status and logs for updates")



from __future__ import absolute_import

import click
from catalyze import cli, client, project, output
from catalyze.helpers import services

@cli.command(short_help = "Execute a rake task")
@click.argument("task_name")
def rake(task_name):
    """Execute a rake task. This is only applicable to ruby-based applications."""
    settings = project.read_settings()
    session = client.acquire_session(settings)
    output.write("Executing Rake Task: {}".format(task_name))
    resp = services.initiate_rake(session, settings["environmentId"], settings["serviceId"], task_name)
    output.write("Rake task output viewable in the logging server.")



from __future__ import absolute_import

from catalyze import config

def list(session, env_id, svc_id):
    route = "%s/v1/environments/%s/services/%s/jobs" % (config.paas_host, env_id, svc_id)
    return session.get(route, verify = True)

def retrieve(session, env_id, svc_id, job_id):
    route = "%s/v1/environments/%s/services/%s/jobs/%s" % (config.paas_host, env_id, svc_id, job_id)
    return session.get(route, verify = True)

def retrieve_brrgc_task(session, env_id, svc_id, task_id):
    route = "%s/v1/environments/%s/services/%s/brrgc/task/%s" % (config.paas_host, env_id, svc_id, task_id)
    return session.get(route, verify = True)

def poll_brrgc_task(session, env_id, svc_id, task_id):
    while True:
        task = retrieve_brrgc_task(session, env_id, svc_id, task_id)
        print(task)
        return task

def poll_until_complete(session, env_id, svc_id, job_id):
    while True:
        job = retrieve(session, env_id, svc_id, job_id)
        if job["status"] not in ["scheduled", "queued", "started", "running"]:
            if job["status"] == "finished":
                return job
            else:
                print("\nError - job ended in status '%s'. Check log for details.")
                sys.exit(1)
            break
        sys.stdout.write(".")
        time.sleep(2)



from __future__ import absolute_import

from catalyze import config, project, output
import requests, json, getpass, os, sys, ssl
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.poolmanager import PoolManager
import requests.packages.urllib3.contrib.pyopenssl
requests.packages.urllib3.contrib.pyopenssl.inject_into_urllib3()

class AuthError(Exception):
    def __init__(self, message):
        Exception.__init__(self, message)

class ClientError(Exception):
    def __init__(self, resp):
        message = resp
        if type(resp) is requests.Response:
            try:
                message = resp.json()
            except ValueError:
                message = {"errors": [{"message": resp.text, "code": -1}]}
        Exception.__init__(self, message)

previous_hook = sys.excepthook

def excepthook(exc_type, value, traceback):
    if exc_type in [AuthError, ClientError]:
        if type(value.message) is dict:
            if "errors" in value.message:
                errors = value.message["errors"]
                if len(errors) == 0:
                    output.error("Unknown error")
                else:
                    for error in errors:
                        if "title" in error:
                            output.error("%(title)s (%(description)s)" % error)
                        else:
                            output.error("%(message)s (%(code)d)" % error)
            elif "title" in value.message and "description" in value.message:
                output.error("%(title)s (%(description)s)" % value.message)
            else:
                output.error(value)
        else:
            output.error(value)
    else:
        previous_hook(exc_type, value, traceback)

sys.excepthook = excepthook

def is_ok(resp):
    return resp.status_code >= 200 and resp.status_code < 300

class ForcedTLSAdapter(HTTPAdapter):
    def init_poolmanager(self, connections, maxsize, block = False):
        self.poolmanager = PoolManager(num_pools = connections,
            maxsize = maxsize,
            block = block,
            ssl_version = ssl.PROTOCOL_TLSv1)

class Session:
    def __init__(self, token = None, user_id = None, username = None, password = None):
        self.session = requests.Session()
        self.session.mount("https://", ForcedTLSAdapter())
        self.session.verify = "skip_cert_validation" not in config.behavior
        if token is None:
            self.sign_in(username, password)
        else:
            self.token = token
            self.user_id = user_id

    def _build_headers(self):
        return {
            "X-Api-Key": config.api_key,
            "Accept": "application/json",
            "Content-Type": "application/json",
            "Authorization": "Bearer " + self.token,
            "X-CLI-Version": config.version
        }

    def get(self, url, verify = False):
        resp = self.session.get(url, headers = self._build_headers())
        if verify:
            if is_ok(resp):
                return None if not resp.text else resp.json()
            else:
                raise ClientError(resp)
        else:
            return resp

    def post(self, url, body, verify = False):
        resp = self.session.post(url, headers = self._build_headers(), data = json.dumps(body))
        if verify:
            if is_ok(resp):
                return None if not resp.text else resp.json()
            else:
                raise ClientError(resp)
        else:
            return resp

    def post_file(self, url, form, verify = False):
        headers = self._build_headers()
        del headers["Content-Type"]
        resp = self.session.post(url, headers = headers, files = form)
        if verify:
            if is_ok(resp):
                return None if not resp.text else resp.json()
            else:
                raise ClientError(resp)
        else:
            return resp

    def put(self, url, body, verify = False):
        resp = self.session.put(url, headers = self._build_headers(), data = json.dumps(body))
        if verify:
            if is_ok(resp):
                return None if not resp.text else resp.json()
            else:
                raise ClientError(resp)
        else:
            return resp

    def delete(self, url, verify = False):
        resp = self.session.delete(url, headers = self._build_headers())
        if verify:
            if is_ok(resp):
                return None if not resp.text else resp.json()
            else:
                raise ClientError(resp)
        else:
            return resp

    def sign_in(self, username, password):
        resp = self.session.post(config.baas_host + "/v2/auth/signin", headers = {
                "X-Api-Key": config.api_key,
                "Accept": "application/json",
                "Content-Type": "application/json"
            }, data = json.dumps({
                "username": username,
                "password": password
            }))
        if is_ok(resp):
            j = resp.json()
            self.token = j["sessionToken"]
            self.user_id = j["usersId"]
        else:
            raise AuthError(resp.json())

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        self.session.close()

def acquire_session(settings = None):
    if settings is not None and "token" in settings and "user_id" in settings:
        session = Session(token = settings["token"], user_id = settings["user_id"])
        resp = session.get(config.baas_host + "/v2/auth/verify")
        if resp.status_code == 200:
            return session
        elif resp.status_code == 401:
            output.write("Session has timed out. Please re-enter credentials.")
    username = os.getenv("CATALYZE_USERNAME") or config.username
    if username is None:
        username = raw_input("Username: ") if "username" not in config.behavior else config.behavior["username"]
    password = os.getenv("CATALYZE_PASSWORD") or config.password
    if password is None:
        password = getpass.getpass("Password: ")
    session = Session(username = username, password = password)
    if settings is not None:
        settings["token"] = session.token
        settings["user_id"] = session.user_id
        project.save_settings(settings)
    return session



from __future__ import absolute_import

import json, click, sys, tty, termios
from select import select
from ws4py.client.threadedclient import WebSocketClient
from catalyze import cli, client, project, output, config
from catalyze.helpers import services

console_closed = False

@cli.command("console", short_help = "Open a secure console to a service")
@click.argument("service_label")
@click.argument("command", required = False, default = None)
def open_console(service_label, command):
    """
Opens a secure console to a code or database service.

For code services, a command is required. This command is executed as root in the context of the application root directory.

For database services, no command is needed - instead, the appropriate command for the database type is run. For example, for a postgres database, psql is run.
"""
    global console_closed
    settings = project.read_settings()
    session = client.acquire_session(settings)

    service_id = services.get_by_label(session, settings["environmentId"], service_label)

    output.write("Opening console to service '%s'" % (service_id))

    task_id = services.request_console(session, settings["environmentId"], service_id, command)["taskId"]

    output.write("Waiting for the console to be ready... This might take a bit.")

    job_id = services.poll_console_job(session, settings["environmentId"], service_id, task_id)
    creds = services.get_console_tokens(session, settings["environmentId"], service_id, job_id)

    try:
        url = creds["url"].replace("http", "ws")
        token = creds["token"]
        output.write("Connecting...")

        sslopt = {}
        if "skip_cert_validation" in config.behavior:
            sslopt["check_hostname"] = False
        ws = ConsoleClient(url, ssl_options = sslopt, headers = [("X-Console-Token", token)])
        ws.daemon = False
        ws.connect()

        with ContextedConsole() as c:
            while not console_closed:
                data = c.get_data()
                if data:
                    ws.send(data)
    finally:
        output.write("Cleaning up")
        services.destroy_console(session, settings["environmentId"], service_id, job_id)

def emulate_console(ws):
    output.write("I am a test")

class ConsoleClient(WebSocketClient):
    def opened(self):
        output.write("Connection opened")

    def closed(self, code, reason):
        global console_closed
        output.write("Connection closed: %s (%s))" % (reason, str(code)))
        console_closed = True

    def received_message(self, message):
        output.write(message, sameline = True)

class ContextedConsole:
    def __enter__(self):
        self._settings = termios.tcgetattr(sys.stdin)
        tty.setcbreak(sys.stdin.fileno())
        return self

    def __exit__(self, type, value, traceback):
        termios.tcsetattr(sys.stdin, termios.TCSADRAIN, self._settings)

    def get_data(self):
        if sys.stdin in select([sys.stdin], [], [], 1)[0]:
            return sys.stdin.read(1)
        return False



from __future__ import absolute_import

import click
from catalyze import cli, client, project, output
from catalyze.helpers import services

@cli.command(short_help = "Start a background worker")
@click.argument("target", default = "worker")#, help = "The name of the Procfile target to invoke as a worker.")
def worker(target):
    """Starts a Procfile target as a worker. Worker containers are intended to be short-lived, one-off tasks."""
    settings = project.read_settings()
    session = client.acquire_session(settings)
    output.write("Initiating a background worker for Service: %s (procfile target = \"%s\")" % (settings['serviceId'], target))
    services.initiate_worker(session, settings["environmentId"], settings["serviceId"], target)
    output.write("Worker started.")



baas_host = "https://api.catalyze.io"
paas_host = "https://paas-api.catalyze.io"
username = None
password = None

dashboard_url = "https://dashboard.catalyze.io"

api_key = "32a384f5-5d11-4214-812e-b35ced9af4d7"

behavior = {}

version = "1.2.3"



from __future__ import absolute_import

from catalyze import config, output

def list(session):
    route = "%s/v1/environments?pageSize=1000" % (config.paas_host,)
    return session.get(route, verify = True)

def retrieve(session, env_id, source = "spec"):
    route = "%s/v1/environments/%s?source=%s" % (config.paas_host, env_id, source)
    return session.get(route, verify = True)

def list_users(session, env_id):
    route = "%s/v1/environments/%s/users" % (config.paas_host, env_id)
    return session.get(route, verify = True)

def add_user(session, env_id, user_id):
    route = "%s/v1/environments/%s/users/%s" % (config.paas_host, env_id, user_id)
    return session.post(route, {}, verify = True)

def remove_user(session, env_id, user_id):
    route = "%s/v1/environments/%s/users/%s" % (config.paas_host, env_id, user_id)
    return session.delete(route, verify = True)



from __future__ import absolute_import

import click
from catalyze import cli, client, git, project, output
from catalyze.helpers import environments, services

@cli.command("support-ids", short_help = "Prints out various helpful IDs.")
def support_ids():
    """Prints out various helpful IDs that can be pasted into support tickets."""
    settings = project.read_settings()
    session = client.acquire_session(settings)
    del settings["token"]
    for pair in settings.items():
        output.write("%s: %s" % pair)


from __future__ import absolute_import

import click
from catalyze import cli, client, project, output
from catalyze.helpers import jobs, environments, services, tasks, pods
import os, os.path, time, sys
import requests
from Crypto import Random
from Crypto.Cipher import AES
import tempfile, shutil, base64, binascii, struct

@cli.group("db", short_help = "Interact with database services")
def db():
    """Interact with database services."""

@db.command("import", short_help = "Imports data into a database")
@click.argument("database_label")
@click.argument("filepath", type=click.Path(exists = True))
@click.option("--mongo-collection", default = None, help = "The name of a specific mongo collection to import into. Only applies for mongo imports.")
@click.option("--mongo-database", default = None, help = "The name of the mongo database to import into, if not using the default. Only applies for mongo imports.")
#@click.option("--postgres-database", default = None, help = "The name of the postgres database to import into, if not using the default. Only applies for postgres imports. This is functionally equivalent to a \"USE <schema>;\" statement.")
#@click.option("--mysql-database", default = None, help = "The name of the mysql database to import into, if not using the default. Only applies for mysql imports. This is functionally equivalent to a \"USE <database>;\" statement."")
@click.option("--wipe-first", is_flag = True, default = False, help = "If set, empties the database before importing. This should not be used lightly.")
def cmd_import(database_label, filepath, mongo_collection, mongo_database, wipe_first, postgres_database = None, mysql_database = None):
    """Imports a file into a chosen database service.

The import is accomplished by encrypting the file and uploading it to Catalyze. An automated service processes the file according to the passed parameters. The command offers the option to either wait until the processing is finished (and be notified of the end result), or to just kick it off.

The type of file depends on the database. For postgres and mysql, this should be a single SQL script with the extension "sql". For mongo, this should be a tar'd, gzipped archive of the dump that you wish to import, with the extension "tar.gz".

If there is an unexpected error, please contact Catalyze support (support@catalyze.io).
"""
    settings = project.read_settings()
    session = client.acquire_session(settings)
    output.write("Looking up service...")
    service_id = services.get_by_label(session, settings["environmentId"], database_label)

    environment = environments.retrieve(session, settings["environmentId"])
    pod = pods.metadata(session, environment["podId"])
    padding_required = pod["importRequiresLength"]

    output.write("Importing '%s' to %s (%s)" % (filepath, database_label, service_id))
    basename = os.path.basename(filepath)
    dir = tempfile.mkdtemp()
    key = Random.new().read(32)
    iv = Random.new().read(AES.block_size)
    output.write("Encrypting...")
    try:
        enc_filepath = os.path.join(dir, basename)
        with open(filepath, 'rb') as file:
            with open(enc_filepath, 'wb') as tf:
                if padding_required:
                    filesize = os.path.getsize(filepath)
                    output.write("File size = %d" % (filesize,))
                    tf.write(struct.pack("<Q", filesize))
                
                contents = file.read()
                contents += b'\0' * (AES.block_size - len(contents) % AES.block_size)
                cipher = AES.new(key, mode = AES.MODE_CBC, IV = iv)
                tf.write(cipher.encrypt(contents))

        with open(enc_filepath, 'rb') as file:
            options = {}
            if mongo_collection is not None:
                options["mongoCollection"] = mongo_collection
            if mongo_database is not None:
                options["mongoDatabase"] = mongo_database
            if postgres_database is not None:
                options["pgDatabase"] = postgres_database
            if mysql_database is not None:
                options["mysqlDatabase"] = mysql_database

            output.write("Uploading...")
            resp = services.initiate_import(session, settings["environmentId"], \
                    service_id, file, \
                    base64.b64encode(binascii.hexlify(key)), \
                    base64.b64encode(binascii.hexlify(iv)), \
                    wipe_first, options)

            task_id = resp["id"]
            output.write("Processing import... (id = %s)" % (task_id,))
            status = tasks.poll_status(session, settings["environmentId"], task_id)
            output.write("\nImport complete (end status = '%s')" % (status,))
    finally:
        shutil.rmtree(dir)



from __future__ import absolute_import

import json, itertools, click

from catalyze import cli, client, project, config, output
from catalyze.helpers import services, environment_variables

@cli.group("vars", short_help = "Check/set/unset environment variables")
def vars_group():
    """Interacts with environment variables for the associated environment."""

@vars_group.command()
def list():
    """List all set variables."""
    settings = project.read_settings()
    session = client.acquire_session(settings)
    service = session.get("%s/v1/environments/%s/services/%s" % (config.paas_host, settings["environmentId"], settings["serviceId"]), verify = True)
    variables = service["data"]["environmentVariables"]
    for value, key in sorted(variables.items()):
        output.write("%s = %s" % (value, key))

@vars_group.command(short_help = "Set or update a variable.")
@click.argument("variables", nargs = -1)
def set(variables):
    """Set or update one or more variables. Expects variables in the form <key>=<value>. Multiple variables can be set at once.

Variable changes will not take effect in the application until it is redeployed (via either a push or 'catalyze redeploy')."""
    settings = project.read_settings()
    session = client.acquire_session(settings)
    body = {}
    for var in variables:
        pieces = var.split("=", 1)
        if len(pieces) != 2:
            output.error("Expected argument form: <key>=<value>")
        else:
            body[pieces[0]] = pieces[1]
    environment_variables.set(session, settings["environmentId"], settings["serviceId"], body)

@vars_group.command()
@click.argument("variable")
def unset(variable):
    """Unset (delete) a variable.

Variable changes will not take effect in the application until it is redeployed (via either a push or 'catalyze redeploy')."""
    settings = project.read_settings()
    session = client.acquire_session(settings)
    environment_variables.unset(session, settings["environmentId"], settings["serviceId"], variable)



from __future__ import absolute_import

from catalyze import cli, client, config, project, output
from catalyze.helpers import environments

@cli.command("environments", short_help = "List your environments")
def list_environments():
    """Lists all environments to which you have access."""
    settings = project.read_settings(required = False)
    session = client.acquire_session(settings)
    envs = environments.list(session)
    for env in envs:
        output.write("%s: %s (state: %s)" % (env["data"]["name"], env["environmentId"], env["state"]))
    if len(envs) == 0:
        output.write("no environments found")



from __future__ import absolute_import

from catalyze import config, output

def metadata(session, pod_id):
    route = "%s/v1/pods/metadata" % (config.paas_host,)
    for pod in session.get(route, verify = True):
        if pod["id"] == pod_id:
            return pod
    output.error("Could not find the pod associated with this environment. Please contact Catalyze support. Please include your environment ID - found via \"catalyze support-ids\"")



from __future__ import absolute_import

from catalyze import config, output
from catalyze.client import ClientError, is_ok
import urllib, json, time

def list(session, env_id):
    route = "%s/v1/environments/%s?source=pod" % (config.paas_host, env_id)
    return session.get(route, verify = True)["data"]["services"]

def initiate_rake(session, env_id, svc_id, task_name):
    route = "%s/v1/environments/%s/services/%s/rake/%s" % \
            (config.paas_host, env_id, svc_id, urllib.quote(task_name, "").replace(" ", "%20"),)
    return session.post(route, {}, verify = True)

def redeploy(session, env_id, svc_id):
    route = "%s/v1/environments/%s/services/%s/redeploy" % (config.paas_host, env_id, svc_id)
    return session.post(route, {}, verify = True)

def get_by_label(session, env_id, label):
    for service in list(session, env_id):
        if service["label"] == label:
            return service["id"]
    output.error("Could not find service with label '%s'" % (label,))

def list_backups(session, env_id, svc_id, page_number, page_size):
    route = "%s/v1/environments/%s/services/%s/backup?pageNum=%d&pageSize=%d" % \
            (config.paas_host, env_id, svc_id, int(page_number), int(page_size))
    return session.get(route, verify = True)

def create_backup(session, env_id, svc_id):
    route = "%s/v1/environments/%s/services/%s/backup" % (config.paas_host, env_id, svc_id)
    body = {
        "archiveType": "cf",
        "encryptionType": "aes"
    }
    return session.post(route, body, verify = True)["taskId"]

def restore_backup(session, env_id, svc_id, backup_id):
    route = "%s/v1/environments/%s/services/%s/restore/%s" % (config.paas_host, env_id, svc_id, backup_id)
    body = {
        "archiveType": "cf",
        "encryptionType": "aes"
    }
    return session.post(route, body, verify = True)["taskId"]

def initiate_import(session, env_id, svc_id, file, key, iv, wipe_first, options):
    parameters = {
        "key": key,
        "iv": iv,
        "wipeBeforeImport": wipe_first,
        "options": options
    }
    route = "%s/v1/environments/%s/services/%s/db/import" % (config.paas_host, env_id, svc_id)
    return session.post_file(route, {"file": file, "parameters": ("parameters.json", json.dumps(parameters), "application/json")}, verify = True)

def initiate_worker(session, env_id, svc_id, target):
    route = "%s/v1/environments/%s/services/%s/background" % (config.paas_host, env_id, svc_id)
    return session.post(route, {
        "target": target
    }, verify = True)

def request_console(session, env_id, svc_id, command = None):
    route = "%s/v1/environments/%s/services/%s/console" % (config.paas_host, env_id, svc_id)
    body = {}
    if command is not None:
        body["command"] = command
    resp = session.post(route, body, verify = False)
    if resp.status_code == 404:
        output.error("This hosting environment does not yet support secure console.")
    elif not is_ok(resp):
        raise ClientError(resp)
    else:
        return resp.json()

def console_job_status(session, env_id, svc_id, task_id):
    route = "%s/v1/environments/%s/services/%s/console/status/%s" % (config.paas_host, env_id, svc_id, task_id)
    return session.get(route, verify = True)

def poll_console_job(session, env_id, svc_id, task_id):
    while True:
        resp = console_job_status(session, env_id, svc_id, task_id)
        if resp["jobId"] is not None:
            return resp["jobId"]
        time.sleep(2)
        output.write(".", sameline = True)

def get_console_tokens(session, env_id, svc_id, job_id):
    route = "%s/v1/environments/%s/services/%s/console/token/%s" % (config.paas_host, env_id, svc_id, job_id)
    return session.get(route, verify = True)

def destroy_console(session, env_id, svc_id, job_id):
    route = "%s/v1/environments/%s/services/%s/console/%s" % (config.paas_host, env_id, svc_id, job_id)
    return session.delete(route, verify = True)



from __future__ import absolute_import

import sys

def write(*args, **kwargs):
    stream = kwargs["stream"] if "stream" in kwargs else sys.stdout
    stream.write(" ".join([str(v) for v in args]) + ("\n" if "sameline" not in kwargs or not kwargs["sameline"] else ""))
    stream.flush()

def error(message, exit = True, exit_code = -1):
    write("ERROR: " + str(message), stream = sys.stderr)
    if exit:
        sys.exit(exit_code)



from setuptools import setup, find_packages

import catalyze.config

with open("requirements.txt", 'r') as file:
    setup(
        name="catalyze",
        version=catalyze.config.version,
        description="CLI for Catalyze Platform-as-a-Service interaction",
        author="Catalyze",
        author_email="support@catalyze.io",
        packages=find_packages(),
        install_requires=file.readlines(),
        include_package_data=True,
        url="https://github.com/catalyzeio/catalyze-paas-cli",
        entry_points={
            "console_scripts": ["catalyze = catalyze:run"]
        }
    )



from __future__ import absolute_import

from catalyze import config

def list(session, env_id, svc_id):
    route = "%s/v1/services/%s/env" % (config.paas_host, svc_id)
    return session.get(route, verify = True)

def set(session, env_id, svc_id, body):
    route = "%s/v1/services/%s/env" % (config.paas_host, svc_id)
    return session.post(route, body, verify = True)

def unset(session, env_id, svc_id, key):
    route = "%s/v1/services/%s/env/%s" % (config.paas_host, svc_id, key)
    return session.delete(route, verify = True)
